from newspaper import Article
import requests
from .models import ArticleFromJson, Articles



class Scraper:
    def __init__(self, api_key: str, mode: str = 'NEWSPAPER'):
        """
        Scraper modes: 
            Newspaper3k => 'NEWSPAPER'
            GNewsAPI => 'GNEWS'
        """
        self.api_key = api_key
        self.mode = mode
        if mode == 'NEWSPAPER':
            self.baseUrl = "https://newsapi.org/v2/everything?"
        elif mode == 'GNEWS':
            self.baseUrl = "https://gnews.io/api/v3/search?"

    def fetch_all_articles(self, query: str, page_size: int = 10) -> Articles:
        """Method to fetch all articles of a specific query.
        Note to get the full body use the get_body method"""
        if self.mode == 'NEWSPAPER':
            url_parameters = "q={0}&pageSize={1}&apiKey={2}".format(
                query, page_size, self.api_key
            )
        elif self.mode == 'GNEWS':
            url_parameters = "q={0}&max={1}&token={2}".format(
                query, page_size, self.api_key
            )
        url = self.baseUrl + url_parameters
        results = requests.get(url).json()
        all_articles = results["articles"]
        return Articles(self.create_article_objects(all_articles))

    def fetch_articles_from_specific_dates(
        self, query: str, date_from: str, date_to: str, page_size: int = 100
    ) -> Articles:
        """Method to fetch articles from specific dates: dates should be in
        format in 2019-08-04"""

        if self.mode == 'NEWSPAPER':
            params = "q={0}&pageSize={1}&apiKey={2}&from={3}&to={4}".format(
                query, page_size, self.api_key, date_from, date_to
            )
        elif self.mode == 'GNEWS': 
            params = "q={0}&max={1}&token={2}&mindate={3}&maxdate={4}".format(
                query, page_size, self.api_key, date_from, date_to
            )
        url = self.baseUrl + params
        results = requests.get(url).json()
        try:
            articles = results["articles"]
        except KeyError:
            return Articles([])
        return Articles(self.create_article_objects(articles))

    def create_article_objects(self, articles) -> list:
        """Helper method to create a list of article objects"""
        list_of_articles = []
        for article in articles:
            body = self.get_body(article["url"])
            if self.mode == 'NEWSPAPER':
                list_of_articles.append(
                    ArticleFromJson(
                        article["source"]["name"],
                        article["title"],
                        article["description"],
                        article["url"],
                        article["publishedAt"],
                        body,
                        article["author"],
                    )
                )
            elif self.mode == 'GNEWS':
                list_of_articles.append(
                    ArticleFromJson(
                        article["source"]["name"],
                        article["title"],
                        article["description"],
                        article["url"],
                        article["publishedAt"],
                        body,
                    )
                )
        return list_of_articles

    def get_body(self, url: str) -> str:
        """Helper method to get the body of a article from its url"""
        try:
            article = Article(url)
            article.download()
            article.parse()
            return article.text  # pragma: no cover
        except Exception:
            return "Could not retrieve body at this time"



import json
import csv
import pickle


class ArticleFromJson:  # pragma: no cover
    """Data model for one article"""

    def __init__(
        self,
        publisher: str,
        title: str,
        description: str,
        url: str,
        date_published,
        content: str,
        author: str = 'Not Found',
    ):
        self.publisher = publisher
        self.title = title
        self.description = description
        self.url = url
        self.date_published = date_published
        self.content = content
        self.author = author

    def __eq__(self, other):
        if not isinstance(other, type(self)):
            return NotImplemented
        return self.__dict__ == other.__dict__


class Articles:  # pragma: no cover
    """Model to contain a list of article data.
    Also has functions to serialize that data"""

    def __init__(self, articles: list):
        self.articles = articles

    def __add__(self, other: 'Articles'):
        articles = self.articles + other.articles
        return Articles(articles)

    def __radd__(self, other: 'Articles'):
        articles = self.articles + other.articles
        return Articles(articles)

    def to_csv(self, csv_name: str):
        """Create a .csv file from the articles data to better visualize"""
        with open(csv_name, "w") as f:
            writer = csv.DictWriter(f, vars(self.articles[0]).keys())
            writer.writeheader()
            for article in self.articles:
                writer.writerow(vars(article))
        f.close()

    def to_json(self):
        """Serializes the article objects to json"""
        article_list = [vars(article) for article in self.articles]
        return json.dumps({"articles": article_list})

    def to_pickle(self, pickle_name: str):
        """Serialization of article objects to byte stream"""
        with open(pickle_name, "wb") as f:
            pickle.dump(self.articles, f)
            f.close()

    def __eq__(self, other):
        if not isinstance(other, type(self)):
            return NotImplemented
        for index, article in enumerate(self.articles):
            if article != other.articles[index]:
                return False
        return True



import sys
import os
import codecs
from setuptools import setup
from os import path


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="newsdatascraper",
    version="0.0.9.4",
    description="Easily query articles",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Erick Torres and Erik Quintanilla",
    author_email="ericktorresdeveloper@gmail.com",
    url="https://github.com/erikqu/NewsDataScraper",
    packages=["newsdatascraper"],
    include_package_data=True,
    install_requires=[
        "requests==2.20.0",
        "httmock==1.3.0",
        "newspaper3k==0.2.8",
        "tests==0.007",
    ],
    license="MIT",
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "Natural Language :: English",
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
    ],
)
